<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2017 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Auth_Adapter_TwoFactor_SMS implements \Zend_Auth_Adapter_Interface
{
    const SESSION_NAMESPACE_NAME = 'DZF_Auth_Adapter_TwoFactor_SMS';

    const SECRET_CODE_EXPIRATION_TIME = 300; // 5 minutes

    /**
     * @var \Zend_Session_Namespace
     */
    protected static $_sessionNamespace;

    /**
     * @var DZF_Sms_Transport_Abstract
     */
    protected static $_smsTransport;

    /**
     * @var string
     */
    protected $_secretCode;

    /**
     * @param   string      $secretCode
     */
    public function __construct($secretCode)
    {
        $this->_secretCode = (string) $secretCode;
    }

    /**
     * @return  \Zend_Auth_Result
     */
    public function authenticate()
    {
        $identity = null;
        $messages = array();

        $sessionSecretCode = self::getSessionSecretCode();

        if (!$sessionSecretCode)
        {
            // session code not set or already expired
            $code = \Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID;
        }
        else if ($this->_secretCode === self::getSessionSecretCode())
        {
            $identity = array('secretCode' => $this->_secretCode);
            $code = \Zend_Auth_Result::SUCCESS;
        }
        else
        {
            $code = \Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID;
        }

        return new \Zend_Auth_Result($code, $identity, $messages);
    }

    /**
     * @return  string
     */
    public static function getSessionSecretCode()
    {
        return (string) self::_getSession()->secretCode;
    }

    /**
     * @param   string          $secretCode
     */
    public static function setSessionSecretCode($secretCode)
    {
        self::_getSession()->secretCode = $secretCode;
        self::_getSession()->setExpirationSeconds(self::SECRET_CODE_EXPIRATION_TIME, 'secretCode');
    }

    /**
     *
     */
    public static function clearSessionSecretCode()
    {
        unset(self::_getSession()->secretCode);
    }

    /**
     * Generate new secret code, save it to the session (configurable) and return.
     *
     * @param   int             $length             (optional, default - 6)
     * @param   boolean         $saveToSession      (optional, default - true)
     * @return  string
     */
    public static function regenerateSecretCode($length = 6, $saveToSession = true)
    {
        $min = (int) ('1' . str_repeat('0', $length - 1));  // 100000
        $max = (int) (str_repeat('9', $length));            // 999999

        $secretCode = (string) mt_rand($min, $max);

        if ($saveToSession)
        {
            self::setSessionSecretCode($secretCode);
        }

        return $secretCode;
    }

    /**
     * @param   string          $phoneNumber
     * @param   string|null     $code
     * @throws  \Exception
     * @return  bool
     */
    public static function sendSmsCode($phoneNumber, $code = null)
    {
        if (!self::$_smsTransport || !(self::$_smsTransport instanceof DZF_Sms_Transport_Abstract))
        {
            throw new \Exception('SMS transport not set!');
        }

        if (!$code)
        {
            $code = self::getSessionSecretCode();

            if (!$code)
            {
                throw new \Exception('Secret code is not specified!');
            }
        }

        $message = 'Your secret code is ' . $code . '. Do not tell it to anyone. ' . \Zend_Registry::get('secure');

        return self::$_smsTransport->sendSms($phoneNumber, $message);
    }

    /**
     * @return  \Zend_Session_Namespace
     */
    protected static function _getSession()
    {
        if (!isset(self::$_sessionNamespace))
        {
            self::$_sessionNamespace = new \Zend_Session_Namespace(self::SESSION_NAMESPACE_NAME);
        }

        return self::$_sessionNamespace;
    }

    /**
     * @param   DZF_Sms_Transport_Abstract      $transport
     */
    public static function setTransport($transport)
    {
        self::$_smsTransport = $transport;
    }
}
