<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2017 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
abstract class DZF_Nagios_Formatter
{
    /**
     * @var string
     */
    protected $_pattern;

    /**
     * @var int
     */
    protected $_mode = DZF_Nagios_Plugin::MODE_UNKNOWN;

    /**
     * Set result string pattern (without final line break).
     *
     * @param   string          $pattern
     * @return  DZF_Nagios_Formatter
     */
    protected function _setPattern($pattern)
    {
        $this->_pattern = $pattern;
        return $this;
    }

    /**
     * @return  string
     * @throws  \Exception
     */
    protected function _getPattern()
    {
        if (!$this->_pattern)
        {
            throw new \Exception('Pattern is not set!');
        }

        return $this->_pattern;
    }

    /**
     * @param   int             $mode       DZF_Nagios_Plugin::MODE_* constant.
     * @return  DZF_Nagios_Formatter
     */
    public function setMode($mode)
    {
        $this->_mode = $mode;
        return $this;
    }

    /**
     * @return  int
     */
    public function getMode()
    {
        return $this->_mode;
    }

    /**
     * Fill pattern with the specified placeholders, not filled parameters will be removed (optional).
     *
     * Threshold signs will be added automatically based on the plugin mode (optional).
     *
     * @param   array           $placeholders
     * @param   boolean         $addThresholdSigns          Whether to add signs for threshold values automatically.
     * @param   string|null     $removeUnfilledWith         Replace all ommited in $placeholders params with the specified value.
     * @return  string
     */
    public function fillPlaceholders($placeholders, $addThresholdSigns = true, $removeUnfilledWith = '')
    {
        $result = $this->_getPattern();
        $result = $this->_preprocessPattern($result);

        foreach ($placeholders as $name => $value)
        {
            if ($addThresholdSigns && $value && ($name === 'threshold_warn' || $name === 'threshold_crit'))
            {
                if ($this->getMode() == DZF_Nagios_Plugin::MODE_HIGHER_IS_BETTER)
                {
                    $value = "{$value}:";
                }
                else if ($this->getMode() == DZF_Nagios_Plugin::MODE_LOWER_IS_BETTER)
                {
                    $value = "~:{$value}";
                }
            }

            $result = str_replace("%{$name}%", $value, $result);
        }

        if (!is_null($removeUnfilledWith))
        {
            $result = $this->_processUnfilled($result);
            $result = preg_replace('/%[a-z_]+%/', $removeUnfilledWith, $result);
        }

        $result = $this->_postprocessResult($result);
        return "{$result}\n";
    }

    /**
     * Preprocess pattern in inheriting classes.
     *
     * @param   string          $string
     * @return  string
     */
    protected function _preprocessPattern($string)
    {
        return $string;
    }

    /**
     * Process result before replacing unfilled placeholders in inheriting classes.
     *
     * @param   string          $string
     * @return  string
     */
    protected function _processUnfilled($string)
    {
        return $string;
    }

    /**
     * Postprocess result after the all manipulations in inheriting classes.
     *
     * @param   string          $string
     * @return  string
     */
    protected function _postprocessResult($string)
    {
        return $string;
    }
}
