<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2017 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */

/**
 * Base nagios plugin structure.
 *
 * Internal parameters:
 *   - mode         -- MODE_HIGHER_IS_BETTER or MODE_LOWER_IS_BETTER, used in status determination and formatters,
 *                     can be set in inheriting classes [optional, default = 'MODE_UNKNOWN']
 *
 * Parameters:
 *   - critical     -- critical threshold value (without sign - will be added automatically depending on plugin mode) [optional, default - '']
 *   - warning      -- warning threshold value (without sign, see 'critical') [optional, default - '']
 *   - min          -- minimal possible value [optional, default - '']
 *   - max          -- maximal possible value [optional, default - '']
 *   - formatter    -- result output formatter (see DZF_Nagios_Formatter_*) [optional, default - 'DZF_Nagios_Formatter_Text']
 *
 * Available default formatters:
 *   DZF_Nagios_Formatter_Text      - simple text message output
 *   DZF_Nagios_Formatter_PerfData  - message with the performance data output
 *   DZF_Nagios_Formatter_LongText  - message, performance data and long text output
 *   @see https://assets.nagios.com/downloads/nagioscore/docs/nagioscore/3/en/pluginapi.html for details
 */
abstract class DZF_Nagios_Plugin
{
    const MODE_UNKNOWN          = 0;
    const MODE_HIGHER_IS_BETTER = 1;
    const MODE_LOWER_IS_BETTER  = 2;

    const VERSION_MAJOR     = 1;
    const VERSION_MINOR     = 0;
    const VERSION_FIX       = 0;

    const STATUS_OK         = 0;
	const STATUS_WARNING    = 1;
	const STATUS_CRITICAL   = 2;
    const STATUS_UNKNOWN    = 3;

    /**
     * Can be changed in inheriting classes.
     *
     * @var array
     */
    protected $_statuses = array(
        self::STATUS_OK         => 'OK',
        self::STATUS_WARNING    => 'WARNING',
        self::STATUS_CRITICAL   => 'CRITICAL',
        self::STATUS_UNKNOWN    => 'UNKNOWN'
    );

    /**
     * Default formatter, can be changed in the inheriting classes.
     *
     * @var string
     */
    protected $_defaultFormatter = 'DZF_Nagios_Formatter_Text';

    /**
     * @var DZF_Nagios_Formatter
     */
    protected $_formatter;

    /**
     * @var int
     */
    protected $_mode = self::MODE_UNKNOWN;

    /**
     * @var mixed
     */
    protected $_warningThreshold = '';

    /**
     * @var mixed
     */
    protected $_criticalThreshold = '';

    /**
     * @var mixed
     */
    protected $_minValue = '';

    /**
     * @var mixed
     */
    protected $_maxValue = '';

    /**
     * @param   DZF_Nagios_Formatter|string         $formatter
     * @return  DZF_Nagios_Plugin
     * @throws  \Exception
     */
    public function setFormatter($formatter)
    {
        if ($formatter instanceof DZF_Nagios_Formatter)
        {
            $this->_formatter = $formatter;
        }
        else if (is_string($formatter))
        {
            $this->_formatter = new $formatter();
        }
        else
        {
            throw new \Exception('Formatter should be the instance of DZF_Nagios_Formatter class or the string.');
        }

        $this->_formatter->setMode($this->_getMode());

        return $this;
    }

    /**
     * If formatter was not set, will be set and returned default one.
     *
     * @return  DZF_Nagios_Formatter
     */
    public function getFormatter()
    {
        if (!$this->_formatter)
        {
            $this->setFormatter($this->_defaultFormatter);
        }

        return $this->_formatter;
    }

    /**
     * @param   mixed       $warningThreshold
     * @return  DZF_Nagios_Plugin
     */
    public function setWarning($warningThreshold)
    {
        $this->_warningThreshold = $warningThreshold;
        return $this;
    }

    /**
     * @return  mixed
     */
    public function getWarning()
    {
        return $this->_warningThreshold;
    }

    /**
     * @param   mixed       $criticalThreshold
     * @return  DZF_Nagios_Plugin
     */
    public function setCritical($criticalThreshold)
    {
        $this->_criticalThreshold = $criticalThreshold;
        return $this;
    }

    /**
     * @return  mixed
     */
    public function getCritical()
    {
        return $this->_criticalThreshold;
    }

    /**
     * @param   mixed       $maxValue
     * @return  DZF_Nagios_Plugin
     */
    public function setMax($maxValue)
    {
        $this->_maxValue = $maxValue;
        return $this;
    }

    /**
     * @return  mixed
     */
    public function getMax()
    {
        return $this->_maxValue;
    }

    /**
     * @param   mixed       $minValue
     * @return  DZF_Nagios_Plugin
     */
    public function setMin($minValue)
    {
        $this->_minValue = $minValue;
        return $this;
    }

    /**
     * @return  mixed
     */
    public function getMin()
    {
        return $this->_minValue;
    }

    /**
     * @param   int         $mode               self::MODE_* constant.
     * @return  DZF_Nagios_Plugin
     */
    protected function _setMode($mode)
    {
        $this->_mode = $mode;
        return $this;
    }

    /**
     * @return  int
     */
    protected function _getMode()
    {
        return $this->_mode;
    }

    /**
     * Calculate the result status using the passed value, plugin mode and threshold values.
     *
     * @param   mixed       $value
     * @return  int
     * @throws  \Exception
     */
    protected function _calculateStatus($value)
    {
        if ($this->_getMode() == self::MODE_HIGHER_IS_BETTER)
        {
            if ($this->getCritical() !== '' && $value <= $this->getCritical())
            {
                return self::STATUS_CRITICAL;
            }
            else if ($this->getWarning() !== '' && $value <= $this->getWarning())
            {
                return self::STATUS_WARNING;
            }
        }
        else if ($this->_getMode() == self::MODE_LOWER_IS_BETTER)
        {
            if ($this->getCritical() !== '' && $value >= $this->getCritical())
            {
                return self::STATUS_CRITICAL;
            }
            else if ($this->getWarning() !== '' && $value >= $this->getWarning())
            {
                return self::STATUS_WARNING;
            }
        }
        else
        {
            throw new \Exception('Wrong mode, can not calculate status!');
        }

        return self::STATUS_OK;
    }

    /**
     * Check status of needed resource and return result string.
     *
     * @param   int         $resultStatus       self::STATUS_* constant.
     * @return  string
     * @throws  \Exception
     */
    public function getResult(&$resultStatus)
    {
        $placeholders = $this->_getResult($resultStatus);

        if (!is_array($placeholders) || !isset($placeholders['message']))
        {
            throw new \Exception('Plugin should return at least the text message!');
        }

        if (!isset($placeholders['status']))
        {
            $placeholders['status'] = $this->_statuses[$resultStatus];
        }

        if (!isset($placeholders['threshold_warn']))
        {
            $placeholders['threshold_warn'] = $this->getWarning();
        }

        if (!isset($placeholders['threshold_crit']))
        {
            $placeholders['threshold_crit'] = $this->getCritical();
        }

        if (!isset($placeholders['min_value']))
        {
            $placeholders['min_value'] = $this->getMin();
        }

        if (!isset($placeholders['max_value']))
        {
            $placeholders['max_value'] = $this->getMax();
        }

        return $this->getFormatter()->fillPlaceholders($placeholders);
    }

    /**
     * Check status of needed resources and return result placeholders.
     *
     * Possible values:
     *  - status
     *  - message (required)
     *  - performance data:
     *    - perfdata_label
     *    - perfdata_value
     *    - perfdata_unit
     *    - threshold_warn
     *    - threshold_crit
     *    - value_min
     *    - value_max
     *  - long_text
     *  - [additional values for custom formatters]
     *
     * @return  array
     */
    abstract protected function _getResult(&$resultStatus);

    /**
     * Get full or partly version information.
     *
     * @param   string|null         $part           (optional, default - null) 'major' || 'minor' || 'fix'
     * @return  string
     */
    public static function getVersion($part = null)
    {
        if (strtolower($part) === 'major')
        {
            return (string) self::VERSION_MAJOR;
        }
        else if (strtolower($part) === 'minor')
        {
            return (string) self::VERSION_MINOR;
        }
        else if (strtolower($part) === 'fix')
        {
            return (string) self::VERSION_FIX;
        }
        else
        {
            return self::VERSION_MAJOR . '.' . self::VERSION_MINOR . '.' . self::VERSION_FIX;
        }
    }
}
