<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2017 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */

/**
 * Plugin for obtaining the count of user registrations since some date.
 *
 * Requirements:
 *   Your database should have the `users` table with the 'created_at' and 'deleted_at' columns
 *   (default values can be changed via parameters).
 *
 * Parameters:
 *   - since                -- minimal 'created_at' date for count [optional, default - 'today']
 *   - only-not-deleted     -- whether to count all the users or only non-deleted ones [optional, default - 'false]
 *   - users-table          -- table to use [optional, default - 'users']
 *   - created-date-column  -- column to use as registration date [optional, default - 'created_at']
 *   - deleted-date-column  -- column to use as deletion date [optional, default - 'deleted_at']
 *   - warning              -- [required] @see DZF_Nagios_Plugin for details
 *   - critical             -- [required] @see DZF_Nagios_Plugin for details
 */
class DZF_Nagios_Plugin_Doctrine_UserRegistrations extends DZF_Nagios_Plugin
{
    const DEFAULT_SINCE = 'today';

    /**
     * @var string 
     */
    protected $_since;

    /**
     * @var boolean
     */
    protected $_onlyNotDeleted = false;

    /**
     * @var string
     */
    protected $_usersTable = 'users';

    /**
     * @var string
     */
    protected $_createdDateColumn = 'created_at';

    /**
     * @var string
     */
    protected $_deletedDateColumn = 'deleted_at';

    /**
     * Initialize the plugin.
     */
    public function __construct() 
    {
        $this->_defaultFormatter = 'DZF_Nagios_Formatter_PerfData';
        $this->_setMode(self::MODE_HIGHER_IS_BETTER);
    }

    /**
     * @param   string      $since          Value, which supported by the strtotime() function.
     * @return  DZF_Nagios_Plugin_Doctrine_UserRegistrations
     */
    public function setSince($since)
    {
        $this->_since = date('Y-m-d 00:00:00', strtotime($since));
        return $this;
    }

    /**
     * Return the 'since' value. If not set, wil be set to default.
     *
     * @return  string
     */
    public function getSince()
    {
        if (!$this->_since)
        {
            $this->setSince(self::DEFAULT_SINCE);
        }

        return $this->_since;
    }

    /**
     * @param   boolean     $notDeleted
     * @return  DZF_Nagios_Plugin_Doctrine_UserRegistrations
     */
    public function setOnlyNotDeleted($notDeleted)
    {
        $this->_onlyNotDeleted = (bool) $notDeleted;
        return $this;
    }

    /**
     * @return  boolean
     */
    public function getOnlyNotDeleted()
    {
        return $this->_onlyNotDeleted;
    }

    /**
     * @param   string      $usersTableName
     * @return  DZF_Nagios_Plugin_Doctrine_UserRegistrations
     */
    public function setUsersTable($usersTableName)
    {
        $this->_usersTable = (string) $usersTableName;
        return $this;
    }

    /**
     * @return  string
     */
    public function getUsersTable()
    {
        return $this->_usersTable;
    }

    /**
     * @param   string      $createdDateColumnName
     * @return  DZF_Nagios_Plugin_Doctrine_UserRegistrations
     */
    public function setCreatedDateColumn($createdDateColumnName)
    {
        $this->_createdDateColumn = $createdDateColumnName;
        return $this;
    }

    /**
     * @return  string
     */
    public function getCreatedDateColumn()
    {
        return $this->_createdDateColumn;
    }

    /**
     * @param   string      $deletedDateColumnName
     * @return  DZF_Nagios_Plugin_Doctrine_UserRegistrations
     */
    public function setDeletedDateColumn($deletedDateColumnName)
    {
        $this->_deletedDateColumn = $deletedDateColumnName;
        return $this;
    }

    /**
     * @return  string
     */
    public function getDeletedDateColumn()
    {
        return $this->_deletedDateColumn;
    }

    /**
     * Count users and return results.
     */
    protected function _getResult(&$resultStatus)
    {
        try
        {
            $query = 'SELECT COUNT(*) FROM `' . $this->getUsersTable() . '` WHERE `' . $this->getCreatedDateColumn() . '` >= ?';
            $queryParams = array($this->getSince());

            if ($this->getOnlyNotDeleted())
            {
                $query .= ' AND `' . $this->getDeletedDateColumn() . '` IS NULL';
            }

            $connection = \Doctrine_Manager::getInstance()->getCurrentConnection();
            $usersCount = $connection->fetchOne($query, $queryParams);

            $status = $this->_calculateStatus($usersCount);
            $message = 'Registrations since ' . $this->getSince() . ': ' . $usersCount;
        }
        catch (\Exception $e)
        {
            $status = self::STATUS_UNKNOWN;
            $message = 'Exception occurred: ' . substr($e->getMessage(), 0, 42) . '...';
            $longText = 'Exception trace:' . "\n" . $e->getTraceAsString();
        }

        $resultStatus = $status;

        return array(
            'message'           => $message,
            'perfdata_label'    => 'Registrations count',
            'perfdata_value'    => isset($usersCount) ? (int) $usersCount : 0,
            'long_text'         => isset($longText) ? (string) $longText : ''
        );
    }
}
